package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CryptoException;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.engines.RSABlindedEngine;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.signers.PSSSigner;
import java.security.AlgorithmParameters;
import java.security.InvalidKeyException;
import java.security.InvalidParameterException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.SignatureSpi;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.MGF1ParameterSpec;
import java.security.spec.PSSParameterSpec;

public class JDKPSSSigner extends SignatureSpi {
   private AlgorithmParameters engineParams;
   private PSSParameterSpec paramSpec;
   private PSSParameterSpec originalSpec;
   private AsymmetricBlockCipher signer;
   private Digest digest;
   private int saltLength;
   private byte trailer;
   private PSSSigner pss;

   private byte getTrailer(int trailerField) {
      if (trailerField == 1) {
         return -68;
      } else {
         throw new IllegalArgumentException("unknown trailer field");
      }
   }

   protected JDKPSSSigner(AsymmetricBlockCipher signer, PSSParameterSpec paramSpec) {
      this.signer = signer;
      if (paramSpec == null) {
         this.originalSpec = null;
         paramSpec = PSSParameterSpec.DEFAULT;
      } else {
         this.originalSpec = paramSpec;
         this.paramSpec = paramSpec;
      }

      this.digest = JCEDigestUtil.getDigest(paramSpec.getDigestAlgorithm());
      this.saltLength = paramSpec.getSaltLength();
      this.trailer = this.getTrailer(paramSpec.getTrailerField());
   }

   protected void engineInitVerify(PublicKey publicKey) {
      if (publicKey instanceof RSAPublicKey) {
         PSSSigner var2;
         PSSSigner var10000 = var2 = new PSSSigner;
         PublicKey var10001 = publicKey;
         JDKPSSSigner var10002 = this;
         PSSSigner var10003 = var2;
         PSSSigner var10004 = var2;
         JDKPSSSigner var10005 = this;
         JDKPSSSigner var10006 = this;
         AsymmetricBlockCipher var3 = this.signer;
         Digest publicKey1 = var10006.digest;
         int var5 = var10005.saltLength;
         var10004.<init>(var3, publicKey1, var5);
         var10002.pss = var10003;
         var10000.init(false, RSAUtil.generatePublicKeyParameter((RSAPublicKey)var10001));
      } else {
         throw new InvalidKeyException("Supplied key is not a RSAPublicKey instance");
      }
   }

   protected void engineInitSign(PrivateKey privateKey, SecureRandom random) {
      if (privateKey instanceof RSAPrivateKey) {
         JDKPSSSigner var10001 = this;
         JDKPSSSigner var10004 = this;
         JDKPSSSigner var10005 = this;
         JDKPSSSigner var10006 = this;
         AsymmetricBlockCipher var6 = this.signer;
         Digest var3 = var10006.digest;
         int var4 = var10005.saltLength;
         byte var5 = var10004.trailer;
         PSSSigner var10000 = var10001.pss = new PSSSigner(var6, var3, var4, var5);
         ParametersWithRandom var7;
         var7 = new ParametersWithRandom.<init>(RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey), random);
         var10000.init(true, var7);
      } else {
         throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
      }
   }

   protected void engineInitSign(PrivateKey privateKey) {
      if (privateKey instanceof RSAPrivateKey) {
         PSSSigner var2;
         PSSSigner var10000 = var2 = new PSSSigner;
         PrivateKey var10001 = privateKey;
         JDKPSSSigner var10002 = this;
         PSSSigner var10003 = var2;
         PSSSigner var10004 = var2;
         JDKPSSSigner var10005 = this;
         JDKPSSSigner var10006 = this;
         JDKPSSSigner var10007 = this;
         AsymmetricBlockCipher var4 = this.signer;
         Digest privateKey1 = var10007.digest;
         int var6 = var10006.saltLength;
         byte var3 = var10005.trailer;
         var10004.<init>(var4, privateKey1, var6, var3);
         var10002.pss = var10003;
         var10000.init(true, RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)var10001));
      } else {
         throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
      }
   }

   protected void engineUpdate(byte b) {
      this.pss.update(b);
   }

   protected void engineUpdate(byte[] b, int off, int len) {
      this.pss.update(b, off, len);
   }

   protected byte[] engineSign() {
      try {
         return this.pss.generateSignature();
      } catch (CryptoException var1) {
         throw new SignatureException(var1.getMessage());
      }
   }

   protected boolean engineVerify(byte[] sigBytes) {
      return this.pss.verifySignature(sigBytes);
   }

   protected void engineSetParameter(AlgorithmParameterSpec params) {
      if (params instanceof PSSParameterSpec) {
         this.paramSpec = (PSSParameterSpec)params;
         PSSParameterSpec params1;
         if ((params1 = this.originalSpec) != null && !JCEDigestUtil.isSameDigest(params1.getDigestAlgorithm(), this.paramSpec.getDigestAlgorithm())) {
            throw new InvalidParameterException("parameter must be using " + this.originalSpec.getDigestAlgorithm());
         } else if (!this.paramSpec.getMGFAlgorithm().equalsIgnoreCase("MGF1") && !this.paramSpec.getMGFAlgorithm().equals(PKCSObjectIdentifiers.id_mgf1.getId())) {
            throw new InvalidParameterException("unknown mask generation function specified");
         } else if (this.paramSpec.getMGFParameters() instanceof MGF1ParameterSpec) {
            MGF1ParameterSpec params2;
            if (JCEDigestUtil.isSameDigest((params2 = (MGF1ParameterSpec)this.paramSpec.getMGFParameters()).getDigestAlgorithm(), this.paramSpec.getDigestAlgorithm())) {
               if ((this.digest = JCEDigestUtil.getDigest(params2.getDigestAlgorithm())) != null) {
                  this.saltLength = this.paramSpec.getSaltLength();
                  this.trailer = this.getTrailer(this.paramSpec.getTrailerField());
               } else {
                  throw new InvalidParameterException("no match on MGF digest algorithm: " + params2.getDigestAlgorithm());
               }
            } else {
               throw new InvalidParameterException("digest algorithm for MGF should be the same as for PSS parameters.");
            }
         } else {
            throw new InvalidParameterException("unkown MGF parameters");
         }
      } else {
         throw new InvalidParameterException("Only PSSParameterSpec supported");
      }
   }

   protected AlgorithmParameters engineGetParameters() {
      if (this.engineParams == null && this.paramSpec != null) {
         Exception var10000;
         label31: {
            boolean var10001;
            AlgorithmParameters var4;
            try {
               var4 = AlgorithmParameters.getInstance("PSS", "INFOSEC");
            } catch (Exception var3) {
               var10000 = var3;
               var10001 = false;
               break label31;
            }

            AlgorithmParameters var1 = var4;

            try {
               this.engineParams = var1;
               var4.init(this.paramSpec);
               return this.engineParams;
            } catch (Exception var2) {
               var10000 = var2;
               var10001 = false;
            }
         }

         Exception var5 = var10000;
         throw new RuntimeException(var5.toString());
      } else {
         return this.engineParams;
      }
   }

   /** @deprecated */
   protected void engineSetParameter(String param, Object value) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   protected Object engineGetParameter(String param) {
      throw new UnsupportedOperationException("engineGetParameter unsupported");
   }

   public static class PSSwithRSA extends JDKPSSSigner {
      public PSSwithRSA() {
         PSSwithRSA var10000 = this;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10000.<init>(var1, (PSSParameterSpec)null);
      }
   }

   public static class SHA1withRSA extends JDKPSSSigner {
      public SHA1withRSA() {
         SHA1withRSA var10000 = this;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10000.<init>(var1, PSSParameterSpec.DEFAULT);
      }
   }

   public static class SHA224withRSA extends JDKPSSSigner {
      public SHA224withRSA() {
         SHA224withRSA var10000 = this;
         RSABlindedEngine var3;
         var3 = new RSABlindedEngine.<init>();
         PSSParameterSpec var1;
         PSSParameterSpec var10001 = var1 = new PSSParameterSpec;
         MGF1ParameterSpec var2;
         var2 = new MGF1ParameterSpec.<init>("SHA-224");
         var10001.<init>("SHA-224", "MGF1", var2, 28, 1);
         var10000.<init>(var3, var1);
      }
   }

   public static class SHA256withRSA extends JDKPSSSigner {
      public SHA256withRSA() {
         SHA256withRSA var10000 = this;
         RSABlindedEngine var3;
         var3 = new RSABlindedEngine.<init>();
         PSSParameterSpec var1;
         PSSParameterSpec var10001 = var1 = new PSSParameterSpec;
         MGF1ParameterSpec var2;
         var2 = new MGF1ParameterSpec.<init>("SHA-256");
         var10001.<init>("SHA-256", "MGF1", var2, 32, 1);
         var10000.<init>(var3, var1);
      }
   }

   public static class SHA384withRSA extends JDKPSSSigner {
      public SHA384withRSA() {
         SHA384withRSA var10000 = this;
         RSABlindedEngine var3;
         var3 = new RSABlindedEngine.<init>();
         PSSParameterSpec var1;
         PSSParameterSpec var10001 = var1 = new PSSParameterSpec;
         MGF1ParameterSpec var2;
         var2 = new MGF1ParameterSpec.<init>("SHA-384");
         var10001.<init>("SHA-384", "MGF1", var2, 48, 1);
         var10000.<init>(var3, var1);
      }
   }

   public static class SHA512withRSA extends JDKPSSSigner {
      public SHA512withRSA() {
         SHA512withRSA var10000 = this;
         RSABlindedEngine var3;
         var3 = new RSABlindedEngine.<init>();
         PSSParameterSpec var1;
         PSSParameterSpec var10001 = var1 = new PSSParameterSpec;
         MGF1ParameterSpec var2;
         var2 = new MGF1ParameterSpec.<init>("SHA-512");
         var10001.<init>("SHA-512", "MGF1", var2, 64, 1);
         var10000.<init>(var3, var1);
      }
   }
}
